@@ -1,6 +1,7 @@
 """Fix up various things after deserialization."""
 
 from typing import Any, Dict, Optional
+from typing_extensions import Final
 
 from mypy.nodes import (
     MypyFile, SymbolNode, SymbolTable, SymbolTableNode,
@@ -10,18 +11,17 @@ from mypy.nodes import (
 from mypy.types import (
     CallableType, Instance, Overloaded, TupleType, TypedDictType,
     TypeVarType, UnboundType, UnionType, TypeVisitor, LiteralType,
-    TypeType, NOT_READY
-)
+    TypeType, NOT_READY, TypeAliasType, AnyType, TypeOfAny)
 from mypy.visitor import NodeVisitor
 from mypy.lookup import lookup_fully_qualified
 
 
-# N.B: we do a quick_and_dirty fixup in both quick_and_dirty mode and
-# when fixing up a fine-grained incremental cache load (since there may
-# be cross-refs into deleted modules)
+# N.B: we do a allow_missing fixup when fixing up a fine-grained
+# incremental cache load (since there may be cross-refs into deleted
+# modules)
 def fixup_module(tree: MypyFile, modules: Dict[str, MypyFile],
-                 quick_and_dirty: bool) -> None:
-    node_fixer = NodeFixer(modules, quick_and_dirty)
+                 allow_missing: bool) -> None:
+    node_fixer = NodeFixer(modules, allow_missing)
     node_fixer.visit_symbol_table(tree.names)
 
 
@@ -29,10 +29,10 @@ def fixup_module(tree: MypyFile, modules: Dict[str, MypyFile],
 class NodeFixer(NodeVisitor[None]):
     current_info = None  # type: Optional[TypeInfo]
 
-    def __init__(self, modules: Dict[str, MypyFile], quick_and_dirty: bool) -> None:
+    def __init__(self, modules: Dict[str, MypyFile], allow_missing: bool) -> None:
         self.modules = modules
-        self.quick_and_dirty = quick_and_dirty
-        self.type_fixer = TypeFixer(self.modules, quick_and_dirty)
+        self.allow_missing = allow_missing
+        self.type_fixer = TypeFixer(self.modules, allow_missing)
 
     # NOTE: This method isn't (yet) part of the NodeVisitor API.
     def visit_type_info(self, info: TypeInfo) -> None:
@@ -57,7 +57,7 @@ class NodeFixer(NodeVisitor[None]):
             if info.metaclass_type:
                 info.metaclass_type.accept(self.type_fixer)
             if info._mro_refs:
-                info.mro = [lookup_qualified_typeinfo(self.modules, name, self.quick_and_dirty)
+                info.mro = [lookup_qualified_typeinfo(self.modules, name, self.allow_missing)
                             for name in info._mro_refs]
                 info._mro_refs = None
         finally:
@@ -74,20 +74,23 @@ class NodeFixer(NodeVisitor[None]):
                     value.node = self.modules[cross_ref]
                 else:
                     stnode = lookup_qualified_stnode(self.modules, cross_ref,
-                                                     self.quick_and_dirty)
+                                                     self.allow_missing)
                     if stnode is not None:
+                        assert stnode.node is not None
                         value.node = stnode.node
-                    elif not self.quick_and_dirty:
-                        assert stnode is not None, "Could not find cross-ref %s" % (cross_ref,)
+                    elif not self.allow_missing:
+                        assert False, "Could not find cross-ref %s" % (cross_ref,)
                     else:
-                        # We have a missing crossref in quick mode, need to put something
-                        value.node = stale_info(self.modules)
+                        # We have a missing crossref in allow missing mode, need to put something
+                        value.node = missing_info(self.modules)
             else:
                 if isinstance(value.node, TypeInfo):
                     # TypeInfo has no accept().  TODO: Add it?
                     self.visit_type_info(value.node)
                 elif value.node is not None:
                     value.node.accept(self)
+                else:
+                    assert False, 'Unexpected empty node %r: %s' % (key, value)
 
     def visit_func_def(self, func: FuncDef) -> None:
         if self.current_info is not None:
@@ -137,9 +140,9 @@ class NodeFixer(NodeVisitor[None]):
 
 
 class TypeFixer(TypeVisitor[None]):
-    def __init__(self, modules: Dict[str, MypyFile], quick_and_dirty: bool) -> None:
+    def __init__(self, modules: Dict[str, MypyFile], allow_missing: bool) -> None:
         self.modules = modules
-        self.quick_and_dirty = quick_and_dirty
+        self.allow_missing = allow_missing
 
     def visit_instance(self, inst: Instance) -> None:
         # TODO: Combine Instances that are exactly the same?
@@ -147,7 +150,7 @@ class TypeFixer(TypeVisitor[None]):
         if type_ref is None:
             return  # We've already been here.
         inst.type_ref = None
-        inst.type = lookup_qualified_typeinfo(self.modules, type_ref, self.quick_and_dirty)
+        inst.type = lookup_qualified_typeinfo(self.modules, type_ref, self.allow_missing)
         # TODO: Is this needed or redundant?
         # Also fix up the bases, just in case.
         for base in inst.type.bases:
@@ -155,8 +158,17 @@ class TypeFixer(TypeVisitor[None]):
                 base.accept(self)
         for a in inst.args:
             a.accept(self)
-        if inst.final_value is not None:
-            inst.final_value.accept(self)
+        if inst.last_known_value is not None:
+            inst.last_known_value.accept(self)
+
+    def visit_type_alias_type(self, t: TypeAliasType) -> None:
+        type_ref = t.type_ref
+        if type_ref is None:
+            return  # We've already been here.
+        t.type_ref = None
+        t.alias = lookup_qualified_alias(self.modules, type_ref, self.allow_missing)
+        for a in t.args:
+            a.accept(self)
 
     def visit_any(self, o: Any) -> None:
         pass  # Nothing to descend into.
@@ -183,6 +195,10 @@ class TypeFixer(TypeVisitor[None]):
         for ct in t.items():
             ct.accept(self)
 
+    def visit_erased_type(self, o: Any) -> None:
+        # This type should exist only temporarily during type inference
+        raise RuntimeError("Shouldn't get here", o)
+
     def visit_deleted_type(self, o: Any) -> None:
         pass  # Nothing to descend into.
 
@@ -199,14 +215,20 @@ class TypeFixer(TypeVisitor[None]):
         if tt.items:
             for it in tt.items:
                 it.accept(self)
-        if tt.fallback is not None:
-            tt.fallback.accept(self)
+        if tt.partial_fallback is not None:
+            tt.partial_fallback.accept(self)
 
     def visit_typeddict_type(self, tdt: TypedDictType) -> None:
         if tdt.items:
             for it in tdt.items.values():
                 it.accept(self)
         if tdt.fallback is not None:
+            if tdt.fallback.type_ref is not None:
+                if lookup_qualified(self.modules, tdt.fallback.type_ref,
+                                    self.allow_missing) is None:
+                    # We reject fake TypeInfos for TypedDict fallbacks because
+                    # the latter are used in type checking and must be valid.
+                    tdt.fallback.type_ref = 'typing._TypedDict'
             tdt.fallback.accept(self)
 
     def visit_literal_type(self, lt: LiteralType) -> None:
@@ -236,22 +258,36 @@ class TypeFixer(TypeVisitor[None]):
 
 
 def lookup_qualified_typeinfo(modules: Dict[str, MypyFile], name: str,
-                              quick_and_dirty: bool) -> TypeInfo:
-    node = lookup_qualified(modules, name, quick_and_dirty)
+                              allow_missing: bool) -> TypeInfo:
+    node = lookup_qualified(modules, name, allow_missing)
     if isinstance(node, TypeInfo):
         return node
     else:
-        # Looks like a missing TypeInfo in quick mode, put something there
-        assert quick_and_dirty, "Should never get here in normal mode," \
-                                " got {}:{} instead of TypeInfo".format(type(node).__name__,
-                                                                        node.fullname() if node
-                                                                        else '')
-        return stale_info(modules)
+        # Looks like a missing TypeInfo during an initial daemon load, put something there
+        assert allow_missing, "Should never get here in normal mode," \
+                              " got {}:{} instead of TypeInfo".format(type(node).__name__,
+                                                                      node.fullname if node
+                                                                      else '')
+        return missing_info(modules)
+
+
+def lookup_qualified_alias(modules: Dict[str, MypyFile], name: str,
+                           allow_missing: bool) -> TypeAlias:
+    node = lookup_qualified(modules, name, allow_missing)
+    if isinstance(node, TypeAlias):
+        return node
+    else:
+        # Looks like a missing TypeAlias during an initial daemon load, put something there
+        assert allow_missing, "Should never get here in normal mode," \
+                              " got {}:{} instead of TypeAlias".format(type(node).__name__,
+                                                                       node.fullname if node
+                                                                       else '')
+        return missing_alias()
 
 
 def lookup_qualified(modules: Dict[str, MypyFile], name: str,
-                     quick_and_dirty: bool) -> Optional[SymbolNode]:
-    stnode = lookup_qualified_stnode(modules, name, quick_and_dirty)
+                     allow_missing: bool) -> Optional[SymbolNode]:
+    stnode = lookup_qualified_stnode(modules, name, allow_missing)
     if stnode is None:
         return None
     else:
@@ -259,18 +295,27 @@ def lookup_qualified(modules: Dict[str, MypyFile], name: str,
 
 
 def lookup_qualified_stnode(modules: Dict[str, MypyFile], name: str,
-                            quick_and_dirty: bool) -> Optional[SymbolTableNode]:
-    return lookup_fully_qualified(name, modules, raise_on_missing=not quick_and_dirty)
+                            allow_missing: bool) -> Optional[SymbolTableNode]:
+    return lookup_fully_qualified(name, modules, raise_on_missing=not allow_missing)
 
 
-def stale_info(modules: Dict[str, MypyFile]) -> TypeInfo:
-    suggestion = "<stale cache: consider running mypy without --quick>"
+_SUGGESTION = "<missing {}: *should* have gone away during fine-grained update>"  # type: Final
+
+
+def missing_info(modules: Dict[str, MypyFile]) -> TypeInfo:
+    suggestion = _SUGGESTION.format('info')
     dummy_def = ClassDef(suggestion, Block([]))
     dummy_def.fullname = suggestion
 
-    info = TypeInfo(SymbolTable(), dummy_def, "<stale>")
+    info = TypeInfo(SymbolTable(), dummy_def, "<missing>")
     obj_type = lookup_qualified(modules, 'builtins.object', False)
     assert isinstance(obj_type, TypeInfo)
     info.bases = [Instance(obj_type, [])]
     info.mro = [info, obj_type]
     return info
+
+
+def missing_alias() -> TypeAlias:
+    suggestion = _SUGGESTION.format('alias')
+    return TypeAlias(AnyType(TypeOfAny.special_form), suggestion,
+                     line=-1, column=-1)
