@@ -11,10 +11,7 @@ import sys
 import tempfile
 
 from typing import Optional, Callable
-
-MYPY = False
-if MYPY:
-    from typing import Type
+from typing_extensions import Final, Type
 
 from types import TracebackType
 
@@ -172,14 +169,13 @@ class IPCClient(IPCBase):
                  exc_ty: 'Optional[Type[BaseException]]' = None,
                  exc_val: Optional[BaseException] = None,
                  exc_tb: Optional[TracebackType] = None,
-                 ) -> bool:
+                 ) -> None:
         self.close()
-        return False
 
 
 class IPCServer(IPCBase):
 
-    BUFFER_SIZE = 2**16
+    BUFFER_SIZE = 2**16  # type: Final
 
     def __init__(self, name: str, timeout: Optional[float] = None) -> None:
         if sys.platform == 'win32':
@@ -205,7 +201,7 @@ class IPCServer(IPCBase):
                                                       )
             if self.connection == -1:  # INVALID_HANDLE_VALUE
                 err = _winapi.GetLastError()
-                raise IPCException('Invalid handle to pipe: {err}'.format(err))
+                raise IPCException('Invalid handle to pipe: {}'.format(err))
         else:
             self.sock_directory = tempfile.mkdtemp()
             sockfile = os.path.join(self.sock_directory, self.name)
@@ -249,7 +245,7 @@ class IPCServer(IPCBase):
                  exc_ty: 'Optional[Type[BaseException]]' = None,
                  exc_val: Optional[BaseException] = None,
                  exc_tb: Optional[TracebackType] = None,
-                 ) -> bool:
+                 ) -> None:
         if sys.platform == 'win32':
             try:
                 # Wait for the client to finish reading the last write before disconnecting
@@ -260,7 +256,6 @@ class IPCServer(IPCBase):
                 DisconnectNamedPipe(self.connection)
         else:
             self.close()
-        return False
 
     def cleanup(self) -> None:
         if sys.platform == 'win32':
