@@ -57,9 +57,8 @@ def test_python_evaluation(testcase: DataDrivenTestCase, cache_dir: str) -> None
         '--no-site-packages',
         '--no-strict-optional',
         '--no-silence-site-packages',
+        '--no-error-summary',
     ]
-    if testcase.name.lower().endswith('_newsemanal'):
-        mypy_cmdline.append('--new-semantic-analyzer')
     py2 = testcase.name.lower().endswith('python2')
     if py2:
         mypy_cmdline.append('--py2')
@@ -92,7 +91,8 @@ def test_python_evaluation(testcase: DataDrivenTestCase, cache_dir: str) -> None
             output.append(line.rstrip("\r\n"))
     if returncode == 0:
         # Execute the program.
-        proc = subprocess.run([interpreter, program], cwd=test_temp_dir, stdout=PIPE, stderr=PIPE)
+        proc = subprocess.run([interpreter, '-Wignore', program],
+                              cwd=test_temp_dir, stdout=PIPE, stderr=PIPE)
         output.extend(split_lines(proc.stdout, proc.stderr))
     # Remove temp file.
     os.remove(program_path)
