@@ -5,17 +5,19 @@ NOTE: These must not be accessed from mypy.nodes or mypy.types to avoid import
       since these may assume that MROs are ready.
 """
 
-from typing import cast, Optional, List, Sequence, Set
+from typing import cast, Optional, List, Sequence, Set, Iterable, TypeVar
+from typing_extensions import Type as TypingType
+import sys
 
 from mypy.types import (
     TupleType, Instance, FunctionLike, Type, CallableType, TypeVarDef, Overloaded,
-    TypeVarType, TypeType, UninhabitedType, FormalArgument, UnionType, NoneType,
+    TypeVarType, UninhabitedType, FormalArgument, UnionType, NoneType, TypedDictType,
     AnyType, TypeOfAny, TypeType, ProperType, LiteralType, get_proper_type, get_proper_types,
-    copy_type
+    copy_type, TypeAliasType, TypeQuery
 )
 from mypy.nodes import (
-    FuncBase, FuncItem, OverloadedFuncDef, TypeInfo, TypeVar, ARG_STAR, ARG_STAR2, Expression,
-    StrExpr
+    FuncBase, FuncItem, OverloadedFuncDef, TypeInfo, ARG_STAR, ARG_STAR2, ARG_POS,
+    Expression, StrExpr, Var
 )
 from mypy.maptype import map_instance_to_supertype
 from mypy.expandtype import expand_type_by_instance, expand_type
@@ -26,52 +28,90 @@ from mypy.typevars import fill_typevars
 from mypy import state
 
 
+def is_recursive_pair(s: Type, t: Type) -> bool:
+    """Is this a pair of recursive type aliases?"""
+    return (isinstance(s, TypeAliasType) and isinstance(t, TypeAliasType) and
+            s.is_recursive and t.is_recursive)
+
+
 def tuple_fallback(typ: TupleType) -> Instance:
     """Return fallback type for a tuple."""
     from mypy.join import join_type_list
 
     info = typ.partial_fallback.type
-    if info.fullname() != 'builtins.tuple':
+    if info.fullname != 'builtins.tuple':
         return typ.partial_fallback
     return Instance(info, [join_type_list(typ.items)])
 
 
+def try_getting_instance_fallback(typ: ProperType) -> Optional[Instance]:
+    """Returns the Instance fallback for this type if one exists.
+
+    Otherwise, returns None.
+    """
+    if isinstance(typ, Instance):
+        return typ
+    elif isinstance(typ, TupleType):
+        return tuple_fallback(typ)
+    elif isinstance(typ, TypedDictType):
+        return typ.fallback
+    elif isinstance(typ, FunctionLike):
+        return typ.fallback
+    elif isinstance(typ, LiteralType):
+        return typ.fallback
+    else:
+        return None
+
+
 def type_object_type_from_function(signature: FunctionLike,
                                    info: TypeInfo,
                                    def_info: TypeInfo,
                                    fallback: Instance,
                                    is_new: bool) -> FunctionLike:
-    # The __init__ method might come from a generic superclass
-    # (init_or_new.info) with type variables that do not map
-    # identically to the type variables of the class being constructed
-    # (info). For example
+    # We first need to record all non-trivial (explicit) self types in __init__,
+    # since they will not be available after we bind them. Note, we use explicit
+    # self-types only in the defining class, similar to __new__ (but not exactly the same,
+    # see comment in class_callable below). This is mostly useful for annotating library
+    # classes such as subprocess.Popen.
+    default_self = fill_typevars(info)
+    if not is_new and not info.is_newtype:
+        orig_self_types = [(it.arg_types[0] if it.arg_types and it.arg_types[0] != default_self
+                            and it.arg_kinds[0] == ARG_POS else None) for it in signature.items()]
+    else:
+        orig_self_types = [None] * len(signature.items())
+
+    # The __init__ method might come from a generic superclass 'def_info'
+    # with type variables that do not map identically to the type variables of
+    # the class 'info' being constructed. For example:
     #
-    #   class A(Generic[T]): def __init__(self, x: T) -> None: pass
-    #   class B(A[List[T]], Generic[T]): pass
+    #   class A(Generic[T]):
+    #       def __init__(self, x: T) -> None: ...
+    #   class B(A[List[T]]):
+    #      ...
     #
-    # We need to first map B's __init__ to the type (List[T]) -> None.
-    signature = bind_self(signature, original_type=fill_typevars(info), is_classmethod=is_new)
-    signature = cast(FunctionLike,
-                     map_type_from_supertype(signature, info, def_info))
+    # We need to map B's __init__ to the type (List[T]) -> None.
+    signature = bind_self(signature, original_type=default_self, is_classmethod=is_new)
+    signature = cast(FunctionLike, map_type_from_supertype(signature, info, def_info))
+
     special_sig = None  # type: Optional[str]
-    if def_info.fullname() == 'builtins.dict':
+    if def_info.fullname == 'builtins.dict':
         # Special signature!
         special_sig = 'dict'
 
     if isinstance(signature, CallableType):
-        return class_callable(signature, info, fallback, special_sig, is_new)
+        return class_callable(signature, info, fallback, special_sig, is_new, orig_self_types[0])
     else:
         # Overloaded __init__/__new__.
         assert isinstance(signature, Overloaded)
         items = []  # type: List[CallableType]
-        for item in signature.items():
-            items.append(class_callable(item, info, fallback, special_sig, is_new))
+        for item, orig_self in zip(signature.items(), orig_self_types):
+            items.append(class_callable(item, info, fallback, special_sig, is_new, orig_self))
         return Overloaded(items)
 
 
 def class_callable(init_type: CallableType, info: TypeInfo, type_type: Instance,
                    special_sig: Optional[str],
-                   is_new: bool) -> CallableType:
+                   is_new: bool, orig_self_type: Optional[Type] = None) -> CallableType:
     """Create a type object type based on the signature of __init__."""
     variables = []  # type: List[TypeVarDef]
     variables.extend(info.defn.type_vars)
@@ -80,22 +120,23 @@ def class_callable(init_type: CallableType, info: TypeInfo, type_type: Instance,
     from mypy.subtypes import is_subtype
 
     init_ret_type = get_proper_type(init_type.ret_type)
+    orig_self_type = get_proper_type(orig_self_type)
     default_ret_type = fill_typevars(info)
+    explicit_type = init_ret_type if is_new else orig_self_type
     if (
-        is_new
-        and isinstance(init_ret_type, (Instance, TupleType))
-        # Only use the return type from __new__ if it is actually returning
-        # a subtype of what we would return otherwise.
-        and is_subtype(init_ret_type, default_ret_type, ignore_type_params=True)
+        isinstance(explicit_type, (Instance, TupleType))
+        # Only use the declared return type from __new__ or declared self in __init__
+        # if it is actually returning a subtype of what we would return otherwise.
+        and is_subtype(explicit_type, default_ret_type, ignore_type_params=True)
     ):
-        ret_type = init_ret_type  # type: Type
+        ret_type = explicit_type  # type: Type
     else:
         ret_type = default_ret_type
 
     callable_type = init_type.copy_modified(
         ret_type=ret_type, fallback=type_type, name=None, variables=variables,
         special_sig=special_sig)
-    c = callable_type.with_name(info.name())
+    c = callable_type.with_name(info.name)
     return c
 
 
@@ -108,8 +149,8 @@ def map_type_from_supertype(typ: Type,
 
     For example, assume
 
-    . class D(Generic[S]) ...
-    . class C(D[E[T]], Generic[T]) ...
+      class D(Generic[S]): ...
+      class C(D[E[T]], Generic[T]): ...
 
     Now S in the context of D would be mapped to E[T] in the context of C.
     """
@@ -130,6 +171,18 @@ def map_type_from_supertype(typ: Type,
     return expand_type_by_instance(typ, inst_type)
 
 
+def supported_self_type(typ: ProperType) -> bool:
+    """Is this a supported kind of explicit self-types?
+
+    Currently, this means a X or Type[X], where X is an instance or
+    a type variable with an instance upper bound.
+    """
+    if isinstance(typ, TypeType):
+        return supported_self_type(typ.item)
+    return (isinstance(typ, TypeVarType) or
+            (isinstance(typ, Instance) and typ != fill_typevars(typ.type)))
+
+
 F = TypeVar('F', bound=FunctionLike)
 
 
@@ -159,11 +212,12 @@ def bind_self(method: F, original_type: Optional[Type] = None, is_classmethod: b
     from mypy.infer import infer_type_arguments
 
     if isinstance(method, Overloaded):
-        return cast(F, Overloaded([bind_self(c, original_type) for c in method.items()]))
+        return cast(F, Overloaded([bind_self(c, original_type, is_classmethod)
+                                   for c in method.items()]))
     assert isinstance(method, CallableType)
     func = method
     if not func.arg_types:
-        # invalid method. return something
+        # Invalid method, return something.
         return cast(F, func)
     if func.arg_kinds[0] == ARG_STAR:
         # The signature is of the form 'def foo(*args, ...)'.
@@ -173,31 +227,35 @@ def bind_self(method: F, original_type: Optional[Type] = None, is_classmethod: b
         # TODO: infer bounds on the type of *args?
         return cast(F, func)
     self_param_type = get_proper_type(func.arg_types[0])
-    if func.variables and (isinstance(self_param_type, TypeVarType) or
-                           (isinstance(self_param_type, TypeType) and
-                            isinstance(self_param_type.item, TypeVarType))):
+    if func.variables and supported_self_type(self_param_type):
         if original_type is None:
-            # Type check method override
-            # XXX value restriction as union?
+            # TODO: type check method override (see #7861).
             original_type = erase_to_bound(self_param_type)
         original_type = get_proper_type(original_type)
 
-        ids = [x.id for x in func.variables]
-        typearg = get_proper_type(infer_type_arguments(ids, self_param_type, original_type)[0])
-        if (is_classmethod and isinstance(typearg, UninhabitedType)
+        all_ids = [x.id for x in func.variables]
+        typeargs = infer_type_arguments(all_ids, self_param_type, original_type,
+                                        is_supertype=True)
+        if (is_classmethod
+                # TODO: why do we need the extra guards here?
+                and any(isinstance(get_proper_type(t), UninhabitedType) for t in typeargs)
                 and isinstance(original_type, (Instance, TypeVarType, TupleType))):
             # In case we call a classmethod through an instance x, fallback to type(x)
-            # TODO: handle Union
-            typearg = get_proper_type(infer_type_arguments(ids, self_param_type,
-                                                           TypeType(original_type))[0])
+            typeargs = infer_type_arguments(all_ids, self_param_type, TypeType(original_type),
+                                            is_supertype=True)
+
+        ids = [tid for tid in all_ids
+               if any(tid == t.id for t in get_type_vars(self_param_type))]
+
+        # Technically, some constrains might be unsolvable, make them <nothing>.
+        to_apply = [t if t is not None else UninhabitedType() for t in typeargs]
 
         def expand(target: Type) -> Type:
-            assert typearg is not None
-            return expand_type(target, {func.variables[0].id: typearg})
+            return expand_type(target, {id: to_apply[all_ids.index(id)] for id in ids})
 
         arg_types = [expand(x) for x in func.arg_types[1:]]
         ret_type = expand(func.ret_type)
-        variables = func.variables[1:]
+        variables = [v for v in func.variables if v.id not in ids]
     else:
         arg_types = func.arg_types[1:]
         ret_type = func.ret_type
@@ -216,6 +274,7 @@ def bind_self(method: F, original_type: Optional[Type] = None, is_classmethod: b
 
 
 def erase_to_bound(t: Type) -> Type:
+    # TODO: use value restrictions to produce a union?
     t = get_proper_type(t)
     if isinstance(t, TypeVarType):
         return t.upper_bound
@@ -253,7 +312,8 @@ def callable_corresponding_argument(typ: CallableType,
 
 
 def make_simplified_union(items: Sequence[Type],
-                          line: int = -1, column: int = -1) -> ProperType:
+                          line: int = -1, column: int = -1,
+                          *, keep_erased: bool = False) -> ProperType:
     """Build union type with redundant union items removed.
 
     If only a single item remains, this may return a non-union type.
@@ -268,15 +328,15 @@ def make_simplified_union(items: Sequence[Type],
 
     Note: This must NOT be used during semantic analysis, since TypeInfos may not
           be fully initialized.
+    The keep_erased flag is used for type inference against union types
+    containing type variables. If set to True, keep all ErasedType items.
     """
-    # TODO: Make this a function living somewhere outside mypy.types. Most other non-trivial
-    #       type operations are not static methods, so this is inconsistent.
     items = get_proper_types(items)
     while any(isinstance(typ, UnionType) for typ in items):
         all_items = []  # type: List[ProperType]
         for typ in items:
             if isinstance(typ, UnionType):
-                all_items.extend(typ.items)
+                all_items.extend(get_proper_types(typ.items))
             else:
                 all_items.append(typ)
         items = all_items
@@ -289,7 +349,7 @@ def make_simplified_union(items: Sequence[Type],
         # Keep track of the truishness info for deleted subtypes which can be relevant
         cbt = cbf = False
         for j, tj in enumerate(items):
-            if i != j and is_proper_subtype(tj, ti):
+            if i != j and is_proper_subtype(tj, ti, keep_erased_types=keep_erased):
                 # We found a redundant item in the union.
                 removed.add(j)
                 cbt = cbt or tj.can_be_true
@@ -411,7 +471,7 @@ def callable_type(fdef: FuncItem, fallback: Instance,
     # TODO: somewhat unfortunate duplication with prepare_method_signature in semanal
     if fdef.info and not fdef.is_static and fdef.arg_names:
         self_type = fill_typevars(fdef.info)  # type: Type
-        if fdef.is_class or fdef.name() == '__new__':
+        if fdef.is_class or fdef.name == '__new__':
             self_type = TypeType.make_normalized(self_type)
         args = [self_type] + [AnyType(TypeOfAny.unannotated)] * (len(fdef.arg_names)-1)
     else:
@@ -423,7 +483,7 @@ def callable_type(fdef: FuncItem, fallback: Instance,
         [None if argument_elide_name(n) else n for n in fdef.arg_names],
         ret_type or AnyType(TypeOfAny.unannotated),
         fallback,
-        name=fdef.name(),
+        name=fdef.name,
         line=fdef.line,
         column=fdef.column,
         implicit=True,
@@ -442,11 +502,48 @@ def try_getting_str_literals(expr: Expression, typ: Type) -> Optional[List[str]]
     2. 'typ' is a LiteralType containing a string
     3. 'typ' is a UnionType containing only LiteralType of strings
     """
-    typ = get_proper_type(typ)
-
     if isinstance(expr, StrExpr):
         return [expr.value]
 
+    # TODO: See if we can eliminate this function and call the below one directly
+    return try_getting_str_literals_from_type(typ)
+
+
+def try_getting_str_literals_from_type(typ: Type) -> Optional[List[str]]:
+    """If the given expression or type corresponds to a string Literal
+    or a union of string Literals, returns a list of the underlying strings.
+    Otherwise, returns None.
+
+    For example, if we had the type 'Literal["foo", "bar"]' as input, this function
+    would return a list of strings ["foo", "bar"].
+    """
+    return try_getting_literals_from_type(typ, str, "builtins.str")
+
+
+def try_getting_int_literals_from_type(typ: Type) -> Optional[List[int]]:
+    """If the given expression or type corresponds to an int Literal
+    or a union of int Literals, returns a list of the underlying ints.
+    Otherwise, returns None.
+
+    For example, if we had the type 'Literal[1, 2, 3]' as input, this function
+    would return a list of ints [1, 2, 3].
+    """
+    return try_getting_literals_from_type(typ, int, "builtins.int")
+
+
+T = TypeVar('T')
+
+
+def try_getting_literals_from_type(typ: Type,
+                                   target_literal_type: TypingType[T],
+                                   target_fullname: str) -> Optional[List[T]]:
+    """If the given expression or type corresponds to a Literal or
+    union of Literals where the underlying values corresponds to the given
+    target type, returns a list of those underlying values. Otherwise,
+    returns None.
+    """
+    typ = get_proper_type(typ)
+
     if isinstance(typ, Instance) and typ.last_known_value is not None:
         possible_literals = [typ.last_known_value]  # type: List[Type]
     elif isinstance(typ, UnionType):
@@ -454,12 +551,123 @@ def try_getting_str_literals(expr: Expression, typ: Type) -> Optional[List[str]]
     else:
         possible_literals = [typ]
 
-    strings = []
+    literals = []  # type: List[T]
     for lit in get_proper_types(possible_literals):
-        if isinstance(lit, LiteralType) and lit.fallback.type.fullname() == 'builtins.str':
+        if isinstance(lit, LiteralType) and lit.fallback.type.fullname == target_fullname:
             val = lit.value
-            assert isinstance(val, str)
-            strings.append(val)
+            if isinstance(val, target_literal_type):
+                literals.append(val)
+            else:
+                return None
         else:
             return None
-    return strings
+    return literals
+
+
+def get_enum_values(typ: Instance) -> List[str]:
+    """Return the list of values for an Enum."""
+    return [name for name, sym in typ.type.names.items() if isinstance(sym.node, Var)]
+
+
+def is_singleton_type(typ: Type) -> bool:
+    """Returns 'true' if this type is a "singleton type" -- if there exists
+    exactly only one runtime value associated with this type.
+
+    That is, given two values 'a' and 'b' that have the same type 't',
+    'is_singleton_type(t)' returns True if and only if the expression 'a is b' is
+    always true.
+
+    Currently, this returns True when given NoneTypes, enum LiteralTypes and
+    enum types with a single value.
+
+    Note that other kinds of LiteralTypes cannot count as singleton types. For
+    example, suppose we do 'a = 100000 + 1' and 'b = 100001'. It is not guaranteed
+    that 'a is b' will always be true -- some implementations of Python will end up
+    constructing two distinct instances of 100001.
+    """
+    typ = get_proper_type(typ)
+    # TODO: Also make this return True if the type is a bool LiteralType.
+    # Also make this return True if the type corresponds to ... (ellipsis) or NotImplemented?
+    return (
+            isinstance(typ, NoneType) or (isinstance(typ, LiteralType) and typ.is_enum_literal())
+            or (isinstance(typ, Instance) and typ.type.is_enum and len(get_enum_values(typ)) == 1)
+    )
+
+
+def try_expanding_enum_to_union(typ: Type, target_fullname: str) -> ProperType:
+    """Attempts to recursively expand any enum Instances with the given target_fullname
+    into a Union of all of its component LiteralTypes.
+
+    For example, if we have:
+
+        class Color(Enum):
+            RED = 1
+            BLUE = 2
+            YELLOW = 3
+
+        class Status(Enum):
+            SUCCESS = 1
+            FAILURE = 2
+            UNKNOWN = 3
+
+    ...and if we call `try_expanding_enum_to_union(Union[Color, Status], 'module.Color')`,
+    this function will return Literal[Color.RED, Color.BLUE, Color.YELLOW, Status].
+    """
+    typ = get_proper_type(typ)
+
+    if isinstance(typ, UnionType):
+        items = [try_expanding_enum_to_union(item, target_fullname) for item in typ.items]
+        return make_simplified_union(items)
+    elif isinstance(typ, Instance) and typ.type.is_enum and typ.type.fullname == target_fullname:
+        new_items = []
+        for name, symbol in typ.type.names.items():
+            if not isinstance(symbol.node, Var):
+                continue
+            new_items.append(LiteralType(name, typ))
+        # SymbolTables are really just dicts, and dicts are guaranteed to preserve
+        # insertion order only starting with Python 3.7. So, we sort these for older
+        # versions of Python to help make tests deterministic.
+        #
+        # We could probably skip the sort for Python 3.6 since people probably run mypy
+        # only using CPython, but we might as well for the sake of full correctness.
+        if sys.version_info < (3, 7):
+            new_items.sort(key=lambda lit: lit.value)
+        return make_simplified_union(new_items)
+    else:
+        return typ
+
+
+def coerce_to_literal(typ: Type) -> ProperType:
+    """Recursively converts any Instances that have a last_known_value or are
+    instances of enum types with a single value into the corresponding LiteralType.
+    """
+    typ = get_proper_type(typ)
+    if isinstance(typ, UnionType):
+        new_items = [coerce_to_literal(item) for item in typ.items]
+        return make_simplified_union(new_items)
+    elif isinstance(typ, Instance):
+        if typ.last_known_value:
+            return typ.last_known_value
+        elif typ.type.is_enum:
+            enum_values = get_enum_values(typ)
+            if len(enum_values) == 1:
+                return LiteralType(value=enum_values[0], fallback=typ)
+    return typ
+
+
+def get_type_vars(tp: Type) -> List[TypeVarType]:
+    return tp.accept(TypeVarExtractor())
+
+
+class TypeVarExtractor(TypeQuery[List[TypeVarType]]):
+    def __init__(self) -> None:
+        super().__init__(self._merge)
+
+    def _merge(self, iter: Iterable[List[TypeVarType]]) -> List[TypeVarType]:
+        out = []
+        for item in iter:
+            out.extend(item)
+        return out
+
+    def visit_type_var(self, t: TypeVarType) -> List[TypeVarType]:
+        return [t]
